package org.etsi.its.adapter;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigInteger;
import java.util.List;
import java.util.Vector;

import org.etsi.ttcn.tci.BooleanValue;
import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tci.RecordOfValue;
import org.etsi.ttcn.tri.TriActionTemplate;
import org.etsi.ttcn.tri.TriAddress;
import org.etsi.ttcn.tri.TriAddressList;
import org.etsi.ttcn.tri.TriComponentId;
import org.etsi.ttcn.tri.TriException;
import org.etsi.ttcn.tri.TriMessage;
import org.etsi.ttcn.tri.TriParameter;
import org.etsi.ttcn.tri.TriParameterList;
import org.etsi.ttcn.tri.TriParameterPassingMode;
import org.etsi.ttcn.tri.TriPortId;
import org.etsi.ttcn.tri.TriPortIdList;
import org.etsi.ttcn.tri.TriSignatureId;
import org.etsi.ttcn.tri.TriStatus;
import org.etsi.ttcn.tri.TriTestCaseId;

import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tri.AbstractPlugin;
import com.testingtech.ttcn.tri.ISAPlugin;
import com.testingtech.ttcn.tri.PluginIdentifier;
import com.testingtech.ttcn.tri.TriAddressImpl;
import com.testingtech.ttcn.tri.TriMessageImpl;
import com.testingtech.ttcn.tri.TriStatusImpl;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.LocalMessage;
import de.tu_berlin.cs.uebb.ttcn.runtime.OctetString;
import de.tu_berlin.cs.uebb.ttcn.tri.TriMessageParameter;

public class TracePortPlugin extends AbstractPlugin implements  ISAPlugin {

	private static final String MSG_ENCODING = "BaseMessages";
	private static final String SIGNATURE_ENCODING = "Signatures";
	private static final long serialVersionUID = 7436181385675637901L;
	private String tracePath = "";
	private RB RB;
	private PluginIdentifier pluginId;
	private TraceParser traceParse;
	private TriComponentId obuComponent;
	private TriPortId obuTsiPortId;
	private TriComponentId rsuComponent;
	private TriPortId rsuTsiPortId;
	private Thread startAnalyzeTraceThread;
	private String powerLevel;
	private String obuName;
	private String dutyCycle;
	private String repNo;
	private byte[] RELEASEMSG;
	
	public TracePortPlugin(PluginIdentifier pluginId){
		this.pluginId = pluginId;
	}


	public void setRB(RB rb) {
		RB = rb;
	}

	public TriStatus setUp() {
		return new TriStatusImpl();
	}

	public TriStatus tearDown() {
		return new TriStatusImpl();
	}

	public TriStatus triCall(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId, TriParameterList parameterList) {
		
		String signatureName = signatureId.getSignatureName();
		if ("setTraceFile".equals(signatureName)) {
			TriParameter tracePathParam = parameterList.get(0);
			String  tracePath = getCharstringParameterData(tracePathParam);
			
			//attaching the trace file to the log file
			File traceFile = new File(tracePath);
			RB.getTciTLProvidedV321TT().tliRT("", System.nanoTime(), "", -1, null, RTLoggingConstants.RT_LOG_INFO, "attach;EMBED;FILE=" + traceFile.getAbsolutePath());

			
			boolean result = setTraceFile(tracePath);
			BooleanReturn(componentId, portId, address, signatureId,
					parameterList, result);
		}else if("getTraceFileList".equals(signatureName)){
			TriParameter tracePathParam = parameterList.get(0);
			String  tracePathStr = getCharstringParameterData(tracePathParam);
			
			File tracePath = new File(tracePathStr);
			if (tracePath.isDirectory()) {
				String[] fileList = tracePath.list();
				Vector<String> traceFiles = new Vector<String>();
				for (int i = 0; i < fileList.length; i++) {
					fileList[i] = tracePath.getAbsolutePath() + File.separator + fileList[i];
					TraceParser parser = new TraceParser(RB, fileList[i]);
					if (parser.checkTraceFile()) {
						traceFiles.add(fileList[i]);
					}
				}
				TraceFileListReturn(componentId, portId, address, signatureId, parameterList, traceFiles.toArray(new String[]{}));
			}
			else if (tracePath.isFile()) {
				TraceParser parser = new TraceParser(RB, tracePathStr);
				if (parser.checkTraceFile()) {
					TraceFileListReturn(componentId, portId, address, signatureId, parameterList, new String []{tracePathStr});
				}
				else {
					TraceFileListReturn(componentId, portId, address, signatureId, parameterList, new String []{});
				}
			}
			else {
				return new TriStatusImpl("Unknown file object! Please verify the path of the trace file or of the trace folder!");
			}
		}else if("getTransactionCnt".equals(signatureName)){
			int traceCnt = getTraceCnt();
			IntegerReturn(componentId, portId, address, signatureId,
					parameterList, traceCnt);
		}else if("startAnalyzeTrace".equals(signatureName)){
			TriParameter transactionCntParam = parameterList.get(0);
			final int transactionCnt = getIntegerParameterData(transactionCntParam);
			startAnalyzeTraceThread = new Thread(tracePath) {

				public void run() {
					startAnalyzeTrace(transactionCnt);
				}
				
			};
			startAnalyzeTraceThread.start();
			BooleanReturn(componentId, portId, address, signatureId,
					parameterList, true);
		}else if("finishAnalyzeTrace".equals(signatureName)){
			//TOD0
		}else if("writeCSVITSParams".equals(signatureName)){
			TriParameter csvFilePathParam = parameterList.get(0);
			String  csvFilePath = getCharstringParameterData(csvFilePathParam);
			
			TriParameter powerLevelParam = parameterList.get(1);
			String  powerLevel = getCharstringParameterData(powerLevelParam);
			
			TriParameter obuNameParam = parameterList.get(2);
			String  obuName = getCharstringParameterData(obuNameParam);
			
			TriParameter dutyCycleParam = parameterList.get(3);
			String  dutyCycle = getCharstringParameterData(dutyCycleParam);
			
			TriParameter repNoParam = parameterList.get(4);
			String  repNo = getCharstringParameterData(repNoParam);
			
			boolean res = writeCSVITSParams(csvFilePath, powerLevel, obuName, dutyCycle, repNo);
			BooleanReturn(componentId, portId, address, signatureId,
					parameterList, res);
		}else if("writeCSVTransaction".equals(signatureName)){
			TriParameter csvFilePathParam = parameterList.get(0);
			String  csvFilePath = getCharstringParameterData(csvFilePathParam);
			
			TriParameter transDurationParam = parameterList.get(1);
			int  transDuration = getIntegerParameterData(transDurationParam);
			
			TriParameter nrOfEmptyMsgParam = parameterList.get(2);
			int  nrOfEmptyMsg = getIntegerParameterData(nrOfEmptyMsgParam);
			
			boolean res = writeCSVTransactionAll(csvFilePath, transDuration, nrOfEmptyMsg);
			BooleanReturn(componentId, portId, address, signatureId,
					parameterList, res);
		}else if("writeCSVMinMaxErrorRate".equals(signatureName)){
			TriParameter csvFilePathParam = parameterList.get(0);
			String  csvFilePath = getCharstringParameterData(csvFilePathParam);
			
			TriParameter minTranDurationParam = parameterList.get(1);
			int  minTranDuration = getIntegerParameterData(minTranDurationParam);
			
			TriParameter minTranIndexParam = parameterList.get(2);
			int  minTranIndex = getIntegerParameterData(minTranIndexParam);
			
			TriParameter maxTranDurationParam = parameterList.get(3);
			int  maxTranDuration = getIntegerParameterData(maxTranDurationParam);
			
			TriParameter maxTranIndexParam = parameterList.get(4);
			int  maxTranIndex = getIntegerParameterData(maxTranIndexParam);
			
			TriParameter averageTranDurationParam = parameterList.get(5);
			int  averageTranDuration = getIntegerParameterData(averageTranDurationParam);
			
			TriParameter nrOfTransactionsParam = parameterList.get(6);
			int  nrOfTransactions = getIntegerParameterData(nrOfTransactionsParam);
			
			TriParameter errorRateParam = parameterList.get(7);
			int  errorRate = getIntegerParameterData(errorRateParam);
			
			boolean res = writeCSVMinMax(csvFilePath, minTranDuration, minTranIndex, maxTranDuration , maxTranIndex, averageTranDuration, nrOfTransactions, errorRate);
			BooleanReturn(componentId, portId, address, signatureId, parameterList, res); 
		}
		
		
		return new TriStatusImpl();
	}
	
	private boolean writeCSVMinMax(String csvFilePath, int minTranDuration, int minTranIndex, int maxTranDuration, int maxTranIndex, int averageTranDuration, int nrOfTransactions, int errorRate){
		boolean result = false;
		String baseFileName = csvFilePath.substring(0, csvFilePath.lastIndexOf("."));
		
		result = writeCSVMinMaxErrorrate(baseFileName, minTranDuration, minTranIndex, maxTranDuration, maxTranIndex, averageTranDuration, nrOfTransactions, errorRate);
		
		return result;
	}
	
	private boolean writeCSVMinMaxErrorrate(String baseFileName, int minTranDuration, int minTranIndex, int maxTranDuration, int maxTranIndex, int averageTranDuration, int nrOfTransactions, int errorRate){
		String fileName = baseFileName + "Summary.csv";
		BufferedWriter bufferedWriter = null;
		try {
            
			File file = new File(fileName);
			
            //Start writing to the output stream
            if (!file.exists()) {
                //Construct the BufferedWriter object
                bufferedWriter = new BufferedWriter(new FileWriter(file, true));
	            bufferedWriter.write("Power Lever");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Obu Name");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Duty Cycle");
	            bufferedWriter.write(";");
	            bufferedWriter.write("RepNo");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Nr of Transactions");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Error Rate");
	            bufferedWriter.write(";");
	            bufferedWriter.write("MinTranIndex");
	            bufferedWriter.write(";");
	            bufferedWriter.write("MinTranDuration");
	            bufferedWriter.write(";");
	            bufferedWriter.write("MaxTranIndex");
	            bufferedWriter.write(";");
	            bufferedWriter.write("MaxTranDuration");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Average Transaction Time");
	            bufferedWriter.newLine();
            }
            if(bufferedWriter == null){
            	 bufferedWriter = new BufferedWriter(new FileWriter(file, true));
            }
            bufferedWriter.write(powerLevel);
            bufferedWriter.write(";");
            bufferedWriter.write(obuName);
            bufferedWriter.write(";");
            bufferedWriter.write(dutyCycle);
            bufferedWriter.write(";");
            bufferedWriter.write(repNo);
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(nrOfTransactions));
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(errorRate));
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(minTranIndex));
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(minTranDuration));
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(maxTranIndex));
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(maxTranDuration));
            bufferedWriter.write(";");
            bufferedWriter.write(Integer.toString(averageTranDuration));
            bufferedWriter.write(";");
            bufferedWriter.newLine();
            
            return true;
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
            return false;
        } catch (IOException ex) {
            ex.printStackTrace();
            return false;
        } finally {
            //Close the BufferedWriter
            try {
                if (bufferedWriter != null) {
                    bufferedWriter.flush();
                    bufferedWriter.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
                return false;
            }
        }
	}
	
	private boolean writeCSVITSParams(String csvFilePath, String powerLevel, String obuName, String  dutyCycle, String repNo){
		boolean result = false;
		String baseFileName = csvFilePath.substring(0, csvFilePath.lastIndexOf("."));
		
		result = writeCSVITSParamsAll(baseFileName,  powerLevel,  obuName,   dutyCycle,  repNo);
		
		return result;
		
	}
	
	private boolean writeCSVITSParamsAll(String baseFileName, String powerLevel, String obuName, String  dutyCycle, String repNo){
		String fileName = baseFileName + "Transactions.csv";
		
		BufferedWriter bufferedWriter = null;
		try {
            
			File file = new File(fileName);
			

            
            //Start writing to the output stream
            if (!file.exists()) {
                //Construct the BufferedWriter object
                bufferedWriter = new BufferedWriter(new FileWriter(file, true));
	            bufferedWriter.write("Power Lever");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Obu Name");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Duty Cycle");
	            bufferedWriter.write(";");
	            bufferedWriter.write("RepNo");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Transaction Duration");
	            bufferedWriter.write(";");
	            bufferedWriter.write("Number of empty UL frames");
	            bufferedWriter.newLine();
            }
            if(bufferedWriter == null){
            	 bufferedWriter = new BufferedWriter(new FileWriter(file, true));
            }
            this.powerLevel = powerLevel;
            this.obuName = obuName;
            this.dutyCycle = dutyCycle;
            this.repNo = repNo;
            
            return true;
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
            return false;
        } catch (IOException ex) {
            ex.printStackTrace();
            return false;
        } finally {
            //Close the BufferedWriter
            try {
                if (bufferedWriter != null) {
                    bufferedWriter.flush();
                    bufferedWriter.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
                return false;
            }
        }
		
	}
		
	private boolean writeCSVTransactionAll(String csvFilePath, int transDuration, int nrOfEmptyMsg){
		String baseFileName = csvFilePath.substring(0, csvFilePath.lastIndexOf("."));
		String fileName = baseFileName + "Transactions.csv";
		 
		BufferedWriter bufferedWriter = null;
	        
	        try {
	            
	            //Construct the BufferedWriter object
	            bufferedWriter = new BufferedWriter(new FileWriter(fileName, true));
	            
	            //Start writing to the output stream
	            bufferedWriter.write(powerLevel);
	            bufferedWriter.write(";");
	            bufferedWriter.write(obuName);
	            bufferedWriter.write(";");
	            bufferedWriter.write(dutyCycle);
	            bufferedWriter.write(";");
	            bufferedWriter.write(repNo);
	            bufferedWriter.write(";");
	            bufferedWriter.write(Integer.toString(transDuration));
	            bufferedWriter.write(";");
	            bufferedWriter.write(Integer.toString(nrOfEmptyMsg));
	            bufferedWriter.newLine();
	            
	            return true;
	        } catch (FileNotFoundException ex) {
	            ex.printStackTrace();
	            return false;
	        } catch (IOException ex) {
	            ex.printStackTrace();
	            return false;
	        } finally {
	            //Close the BufferedWriter
	            try {
	                if (bufferedWriter != null) {
	                    bufferedWriter.flush();
	                    bufferedWriter.close();
	                }
	            } catch (IOException ex) {
	                ex.printStackTrace();
		            return false;
	            }
	        }
	}
	


	private void IntegerReturn(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId,
			TriParameterList parameterList, int traceCnt) {
		IntegerValue iv = (IntegerValue) RB.getTciCDRequired().getInteger().newInstance();
		iv.setInt(traceCnt);
		TriMessage returnValue = RB.TestAdapter.getCodec(SIGNATURE_ENCODING).encode(iv);
		RB.getTriCommunicationTE().triEnqueueReply(portId, address, componentId, signatureId, parameterList, new TriMessageParameter(RB, TriParameterPassingMode.TRI_IN, "", returnValue));
	}

	private void BooleanReturn(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId,
			TriParameterList parameterList, boolean result) {
		BooleanValue bv = (BooleanValue) RB.getTciCDRequired().getBoolean().newInstance();
		bv.setBoolean(result);
		TriMessage returnValue = RB.TestAdapter.getCodec(SIGNATURE_ENCODING).encode(bv);
		RB.getTriCommunicationTE().triEnqueueReply(portId, address, componentId, signatureId, parameterList, new TriMessageParameter(RB, TriParameterPassingMode.TRI_IN, "", returnValue));
	}

	private void TraceFileListReturn(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId,
			TriParameterList parameterList, String [] result) {
		RecordOfValue rov = (RecordOfValue) RB.getTciCDRequired().getTypeForName("TraceParserSystem.TraceFileList").newInstance();
		rov.setLength(0);
		for (String traceFileStr : result) {
			CharstringValue cv = (CharstringValue) RB.getTciCDRequired().getCharstring().newInstance();
			cv.setString(traceFileStr);
			rov.appendField(cv);
		}
		TriMessage returnValue = RB.TestAdapter.getCodec(SIGNATURE_ENCODING).encode(rov);
		RB.getTriCommunicationTE().triEnqueueReply(portId, address, componentId, signatureId, parameterList, new TriMessageParameter(RB, TriParameterPassingMode.TRI_IN, "", returnValue));
	}

	private boolean setTraceFile(String tracePathPr){
		 File traceFile = new File(tracePathPr);
		
		try {
			boolean fileCreated = traceFile.createNewFile();
			if (!fileCreated && traceFile.canRead()) {
				this.tracePath = tracePathPr;
				return true;
			}
			if (fileCreated) {
				traceFile.delete();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return false;
	}

	private int getTraceCnt(){	
		traceParse = new TraceParser(RB, tracePath);
		traceParse.parseTrace();
		
		return traceParse.getTraceCount();

	}
	
	private boolean startAnalyzeTrace(int trace){
        boolean result = false;
		List<TransactionStructure> transactions = traceParse.getTrace(trace);
		TriPortId tsiPortId = null;
		TriComponentId componentId = null;
		boolean firstUL=true;
		OctetString releaseEvent=null;
		BigInteger lastUL_TS=null;
		
		for (TransactionStructure transaction : transactions) {
			if (transaction.getDirection() == TransactionStructure.OBU) { //UL
				tsiPortId = obuTsiPortId;
				componentId = obuComponent;
				// if the first UL was registered then take the LinkId and build the Release Msg
				if (firstUL && transaction.getPayload() != null && transaction.getPayload().length>=4) {
					OctetString linkId = new OctetString(RB, transaction.getPayload().length, transaction.getPayload()).substr(0, 4);
					RELEASEMSG = new byte[]{(byte)0x80, (byte)0x03 , (byte)0x91 , (byte)0x20 , (byte)0x00, (byte)0x00};
					releaseEvent = linkId.concat(new OctetString(RB, RELEASEMSG.length, RELEASEMSG));
				}
				//take the time stamp of the last UL in order to calculate later the transaction duration
				lastUL_TS = transaction.getTimeStamp();
			}
			if (transaction.getDirection() == TransactionStructure.RSU) { //DL
				tsiPortId = rsuTsiPortId;
				componentId = rsuComponent;
			}
			byte[] payload = transaction.getPayload();
			if (payload==null) {
				payload = new byte[]{};//TODO for empty messages it is null instead of empty array
			}
			RB.getTriCommunicationTE().triEnqueueMsg(tsiPortId, getAddress(transaction.getTimeStamp()), componentId, new TriMessageImpl(payload));
			// This sleep is added as a workaround so that the exchanged of the messages between the components is shoed synchronized
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				//ignore
			}
		}
		// at the end of transaction enqueue the Release Msg
		if (releaseEvent!=null && lastUL_TS!=null) {
			RB.getTriCommunicationTE().triEnqueueMsg(tsiPortId, getAddress(lastUL_TS), componentId, new TriMessageImpl(releaseEvent.toByteArray()));
		}
		return result;
	}


	private TriAddress getAddress(BigInteger timestamp) {
		IntegerValue iv = (IntegerValue) RB.getTciCDRequired().getInteger().newInstance();
		iv.setBigInt(timestamp);
		TriMessage address = RB.TestAdapter.getCodec(MSG_ENCODING).encode(iv);
		return new TriAddressImpl(address.getEncodedMessage());
	}
	
	public String getCharstringParameterData(final TriParameter param) throws UnsupportedOperationException {
		if (param instanceof TriMessageParameter) {
			TriMessageParameter triMsgParam = (TriMessageParameter) param;
			if (triMsgParam.getMessage() instanceof LocalMessage) {
				LocalMessage localMsg = (LocalMessage) triMsgParam.getMessage();
				if (localMsg.getContent() instanceof CharstringValue) {
					CharstringValue v = (CharstringValue) localMsg.getContent();
					if (!v.notPresent()) {
						return v.getString();
					}
					else {
						return null;
					}
				}
			}
		}
		throw new UnsupportedOperationException("getCharstringParameterData: Parameter not of type charstring");
	}
	
	public int getIntegerParameterData(final TriParameter param) throws UnsupportedOperationException {
		if (param instanceof TriMessageParameter) {
			TriMessageParameter triMsgParam = (TriMessageParameter) param;
			if (triMsgParam.getMessage() instanceof LocalMessage) {
				LocalMessage localMsg = (LocalMessage) triMsgParam.getMessage();
				if (localMsg.getContent() instanceof IntegerValue) {
					IntegerValue v = (IntegerValue) localMsg.getContent();
					if (!v.notPresent()) {
						return v.getInt();
					}
				}
			}
		}
		throw new UnsupportedOperationException("getIntegerParameterData: Parameter not of type integer or omit");

	}

	public TriStatus triCallBC(TriComponentId componentId, TriPortId portId,
			TriSignatureId signatureId, TriParameterList parameterList) {
		return new TriStatusImpl("triCallBC not supported");
	}


	public TriStatus triCallMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriSignatureId signatureId, TriParameterList parameterList) {
		return new TriStatusImpl("triCallMC not supported");
	}


	public TriStatus triEndTestCase() {
		if (startAnalyzeTraceThread!=null && startAnalyzeTraceThread.isAlive()) {
			try {
				startAnalyzeTraceThread.join(1000);
			} catch (InterruptedException e) {
				//ignore
			}
		}
		return new TriStatusImpl();
	}


	public TriStatus triExecuteTestcase(TriTestCaseId testCaseId, TriPortIdList portIdList) {
		return new TriStatusImpl();
	}


	public TriStatus triMap(TriPortId compPortId, TriPortId tsiPortId) {
		if (compPortId.getPortName().equals("obuPort")) {
			obuComponent = compPortId.getComponent();
			obuTsiPortId = tsiPortId;
		}
		else if (compPortId.getPortName().equals("rsuPort")) {
			rsuComponent = compPortId.getComponent();
			rsuTsiPortId = tsiPortId;
		}
		else if (compPortId.getPortName().equals("controlPort")) {
			
		}
		else {
			return new TriStatusImpl("Unsupported port: " + compPortId.getPortName());
		}
		return new TriStatusImpl();
	}


	public TriStatus triRaise(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId, TriException exception) {
		return new TriStatusImpl("triRaise not supported");
	}


	public TriStatus triRaiseBC(TriComponentId componentId, TriPortId portId,
			TriSignatureId signatureId, TriException exception) {
		return new TriStatusImpl("triRaiseBC not supported");
	}


	public TriStatus triRaiseMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriSignatureId signatureId, TriException exception) {
		return new TriStatusImpl("triRaiseMC not supported");
	}


	public TriStatus triReply(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId, TriParameterList parameterList,
			TriParameter parameter) {
		return new TriStatusImpl("triReply not supported");
	}


	public TriStatus triReplyBC(TriComponentId componentId, TriPortId portId,
			TriSignatureId signatureId, TriParameterList parameterList, TriParameter parameter) {
		return new TriStatusImpl("triReplyBC not supported");
	}


	public TriStatus triReplyMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriSignatureId signatureId, TriParameterList parameterList,
			TriParameter parameter) {
		return new TriStatusImpl("triReplyMC not supported");
	}


	public TriStatus triSAReset() {
		return new TriStatusImpl();
	}


	public TriStatus triSend(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriMessage message) {
		return new TriStatusImpl("triSend not supported");
	}


	public TriStatus triSendBC(TriComponentId componentId, TriPortId portId,
			TriMessage message) {
		return new TriStatusImpl("triSendBC not supported");
	}


	public TriStatus triSendMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriMessage message) {
		return new TriStatusImpl("triSendMC not supported");
	}


	public TriStatus triSutActionInformal(String str) {
		return new TriStatusImpl("triSutActionInformal not supported");
	}


	public TriStatus triSutActionTemplate(TriActionTemplate actionTemplate) {
		return new TriStatusImpl("triSutActionTemplate not supported");
	}


	public TriStatus triUnmap(TriPortId portId, TriPortId portId1) {
		obuComponent = null;
		obuTsiPortId = null;
		rsuComponent = null;
		rsuTsiPortId = null;
		return new TriStatusImpl();
	}

}
